//-------------------------------------------------------------------
// AD_speed_test.cpp
//
// Tests the conversion speed of the ADSmartIO and UCB1200 A/Ds
//
//
//  8may02 20-11301 ADS/ak
// 31oct02 20-11302 ADS/ak
// - add test that adds toggling of a UCB digital I/O
// - shorten multi-channel tests to be same duration as single-chan
//-------------------------------------------------------------------

#include "stdafx.h"
#include "ports.h"
#define WIN32_LEAN_AND_MEAN
#include <windows.h>

#define NUM_ITERATIONS 24

// Declare library for dynamic linking to SMARTIO.DLL
HINSTANCE gLibSMARTIO = NULL;

// Declare library for dynamic linking to KEYBDDR.DLL
HINSTANCE gLibKEYBDDR = NULL;

typedef UINT (*SIOANALOGCONVERSION)(BYTE channel);
typedef void (*SIOSELECTOPTION)(BYTE Option);
typedef void (*SIOSETSSPTIMEOUT)(UINT Option);
typedef void (*SIOINITDDRA)(BYTE value);
typedef void (*SIOINITDDRB)(BYTE value);
typedef void (*SIOINITDDRC)(BYTE value);
typedef void (*SIOINITDDRD)(BYTE value);
typedef UINT (*SIOREADPORTA)(void);
typedef UINT (*SIOREADPORTB)(void);
typedef UINT (*SIOREADPORTC)(void);
typedef UINT (*SIOREADPORTD)(void);
 

SIOANALOGCONVERSION		SIOAnalogConversion;
SIOSELECTOPTION			SIOSelectOption;
SIOSETSSPTIMEOUT		SIOSetSSPTimeout;
SIOINITDDRA				SIOInitDDRA;


//-------------------------------------------------------------------
//-------------------------------------------------------------------
BOOL LoadSmartIO()
{
	gLibSMARTIO = LoadLibrary((L"SMARTIO.dll"));

	if(gLibSMARTIO == NULL)
	{	
		gLibKEYBDDR = LoadLibrary((L"KEYBDDR.dll"));

		if(gLibKEYBDDR == NULL)
		{	
			RETAILMSG(TRUE,(L"!Failed to load ADSmartIO library.\r\n"));
			return FALSE;
		}
		else
		{	
			DEBUGMSG(TRUE,(L"ADSmartIO is using KEYBDDR.DLL\r\n"));
			gLibSMARTIO = gLibKEYBDDR;
			//GraphicsClientPlus = 1;
		}
	}
	else
	{	
		DEBUGMSG(TRUE,(L"ADSmartIO is using SMARTIO.DLL\r\n"));
		// GraphicsMaster = 1;
	}

	if(gLibSMARTIO!=NULL)
	{	
		SIOAnalogConversion = (SIOANALOGCONVERSION)GetProcAddress(gLibSMARTIO,(_T("SIOAnalogConversion")));
		SIOSelectOption = (SIOSELECTOPTION)GetProcAddress(gLibSMARTIO,(L"SIOSelectOption"));
		SIOSetSSPTimeout = (SIOSETSSPTIMEOUT)GetProcAddress(gLibSMARTIO,(L"SIOSetSSPTimeout"));
		SIOInitDDRA = (SIOINITDDRA)GetProcAddress(gLibSMARTIO,(L"SIOInitDDRA"));
	}

	return TRUE;

}

//-------------------------------------------------------------------
// Reads A/D channel nreps times.
// Returns the number of ticks (ms) it took to make all the readings.
//-------------------------------------------------------------------
DWORD test_sio_AD (byte channel, int nreps)
{

  DWORD dwStart, dwFinish;

  dwStart=GetTickCount();

  for (int i=0;i<nreps;i++)
    SIOAnalogConversion(channel);

  dwFinish=GetTickCount();

  return (dwFinish-dwStart);
}

//-------------------------------------------------------------------
//-------------------------------------------------------------------
DWORD test_all_sio_AD (int nreps)
{

  DWORD dwStart, dwFinish;

  dwStart=GetTickCount();

  for (int i=0;i<nreps;i++)
    for (int j=0;j<8;j++)
      SIOAnalogConversion(j);

  dwFinish=GetTickCount();

  return (dwFinish-dwStart);
}

//-------------------------------------------------------------------
//-------------------------------------------------------------------
DWORD test_ucb_AD1 (int nreps)
{
  HANDLE hPort;
  DWORD dwStart, dwFinish, dwData, dwCount;

  if (!Open_Port(hPort, L"AIN1:", FALSE))
    return 0;

  dwStart=GetTickCount();

  for (int i=0;i<nreps;i++)
    ReadFile(hPort, &dwData, 2, &dwCount, NULL);  // don't care about return data

  dwFinish=GetTickCount();

  Close_Port(hPort);

  return (dwFinish-dwStart);
}

//-------------------------------------------------------------------
//-------------------------------------------------------------------
DWORD test_ucb_AD3 (int nreps)
{
  HANDLE hPort;
  DWORD dwStart, dwFinish, dwData, dwCount;

  if (!Open_Port(hPort, L"AIN3:", FALSE))
    return 0;

  dwStart=GetTickCount();

  for (int i=0;i<nreps;i++)
    ReadFile(hPort, &dwData, 2, &dwCount, NULL);  // don't care about return data

  dwFinish=GetTickCount();

  Close_Port(hPort);

  return (dwFinish-dwStart);
}

//-------------------------------------------------------------------
//-------------------------------------------------------------------
DWORD test_ucb_AD1to4 (int nreps)
{
  HANDLE hPort[4];
  DWORD dwStart, dwFinish, dwData, dwCount;
  int i,j;

  if (!Open_Port(hPort[0], L"AIN1:", FALSE))
    return 0;
  if (!Open_Port(hPort[1], L"AIN2:", FALSE))
    return 0;
  if (!Open_Port(hPort[2], L"AIN3:", FALSE))
    return 0;
  if (!Open_Port(hPort[3], L"AIN4:", FALSE))
    return 0;

  dwStart=GetTickCount();

  for (i=0;i<nreps;i++)
    for (j=0;j<4;j++)
      ReadFile(hPort[j], &dwData, 2, &dwCount, NULL);  // don't care about return data

  dwFinish=GetTickCount();

  for (j=0;j<4;j++)
    Close_Port(hPort[j]);

  return (dwFinish-dwStart);
}

//-------------------------------------------------------------------
// This test turns on UCB1200 digital port 1, reads an A/D channel,
// then turns the digital port off.
//-------------------------------------------------------------------
DWORD test_ucb_Analog_and_IO (int nreps)
{
  HANDLE hPort[4], hIOPort;
  DWORD dwStart, dwFinish, dwData, dwCount;
  DWORD dwDataStart=1, dwDataEnd=0;
  int i,j;

  if (!Open_Port(hPort[0], L"AIN1:", FALSE))
    return 0;
  if (!Open_Port(hPort[1], L"AIN2:", FALSE))
    return 0;
  if (!Open_Port(hPort[2], L"AIN3:", FALSE))
    return 0;
  if (!Open_Port(hPort[3], L"AIN4:", FALSE))
    return 0;
  if (!Open_Port(hIOPort, L"IOA1:", FALSE))
    return 0;

  dwStart=GetTickCount();

  for (i=0;i<nreps;i++)
    for (j=0;j<4;j++)
	{
	  WriteFile(hIOPort, &dwDataStart, 1, &dwCount, NULL);  // signal start of A/D
      ReadFile(hPort[j], &dwData, 2, &dwCount, NULL);       // read data
	  WriteFile(hIOPort, &dwDataEnd, 1, &dwCount, NULL);    // signal start of A/D
	}

  dwFinish=GetTickCount();

  Close_Port (hIOPort);
  for (j=0;j<4;j++)
    Close_Port(hPort[j]);

  return (dwFinish-dwStart);
}

//-------------------------------------------------------------------
//-------------------------------------------------------------------
int WINAPI WinMain(	HINSTANCE hInstance,
					HINSTANCE hPrevInstance,
					LPTSTR    lpCmdLine,
					int       nCmdShow)
{
  // 0. Initialize
  int iteration=0;
  int nreps_ucb, nreps_sio;
  int originalPriority, newPriority, currentPriority=-1;
  HANDLE hThread;

  RETAILMSG(TRUE,(L"\r\nAD_speed_test 20-11302: Tests conversion speed of ADSmartIO and UCB A/D inputs.\r\n"));

	if (!LoadSmartIO())
  {
    RETAILMSG(TRUE,(L"Program aborted.\r\n"));
    return(1);
  }

  nreps_sio = 100;
  nreps_ucb = 10000;
  RETAILMSG(TRUE, (_T("Each ADSMARTIO channel tested %d times.\r\n"),nreps_sio));
  RETAILMSG(TRUE, (_T("Each UCB1200 channel tested %d times.\r\n"),nreps_ucb));

  SIOSelectOption(2);
  SIOInitDDRA(0x00);
  SIOSetSSPTimeout(150);
  RETAILMSG(TRUE, (_T("ADSMARTIO: Option 2, DDRA=0x00(inputs), SSPtimeout=150\r\n\n"),nreps_ucb));

  RETAILMSG(TRUE, (_T("                       Time to read channels (ms)          \r\n")));
  RETAILMSG(TRUE, (_T("Pass #/  ------ ADSMARTIO -----   ------------- UCB1200 -------------\r\n")));
  RETAILMSG(TRUE, (_T("Priority     PA0     PA5   PA0-7    AIN1    AIN3  AIN1to4 +I/O_toggle \r\n")));
  //                   xxx  xxx  xxxxxx  xxxxxx  xxxxxx  xxxxxx  xxxxxx  xxxxxxx    xxxxxx
  //                   123 2345 2345678 2345678 2345678 2345678 2345678 23456789 234567890

  // get current thread priority 
  hThread = GetCurrentThread();
  originalPriority = CeGetThreadPriority(hThread);

  // 2. Run tests
	while (iteration<NUM_ITERATIONS)
	{
		// 2.0 Set and display current thread priority
		// perform 3 tests @ std priority, 3 @ top priority
		if ((iteration%6) < 3)
			// set thread to standard priority 
			newPriority = originalPriority;
		else
			// set thread to standard priority 
			newPriority = 0;
		if (currentPriority != newPriority)
		{
			currentPriority = newPriority;
			CeSetThreadPriority(hThread, currentPriority);
		}

		RETAILMSG(TRUE, (_T("%3d%5d"),iteration+1,currentPriority));

		// 2.1 ADSmartIO PA0
		RETAILMSG(TRUE, (_T("%8d"),test_sio_AD(0,nreps_sio)));
		Sleep(50);  // yield to other processes

		// 2.2 ADSmartIO PA5
		RETAILMSG(TRUE, (_T("%8d"),test_sio_AD(5,nreps_sio)));
		Sleep(50);  // yield to other processes

		// 2.3 ADSmartIO PA0 through PA7
		RETAILMSG(TRUE, (_T("%8d"),test_all_sio_AD((int)(nreps_sio/8))));
		Sleep(50);  // yield to other processes

		// 2.4 UCB1200 AIN1:
		RETAILMSG(TRUE, (_T("%8d"),test_ucb_AD1(nreps_ucb)));
		Sleep(50);  // yield to other processes

		// 2.5 UCB1200 AIN3:
		RETAILMSG(TRUE, (_T("%8d"),test_ucb_AD3(nreps_ucb)));
		Sleep(50);  // yield to other processes

		// 2.6 UCB1200 AIN1: through AIN4:
		RETAILMSG(TRUE, (_T("%9d"),test_ucb_AD1to4((int)(nreps_ucb/4))));
		Sleep(50);  // yield to other processes

		// 2.7 UCB1200 AIN1: through AIN4: toggling I/O before and after each test
		RETAILMSG(TRUE, (_T("%10d"),test_ucb_Analog_and_IO((int)(nreps_ucb/4))));
		Sleep(50);  // yield to other processes

		RETAILMSG(TRUE, (_T("   \r\n")));

		iteration++;
  }

  // 3. Shut down

  RETAILMSG(TRUE,(L"\r\nA2D_speed_test: Complete.\r\n"));

	return 0;

}

